#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const fs_1 = require("fs");
const globby_1 = require("globby");
const normalizePath = require("normalize-path");
const path_1 = require("path");
const util_1 = require("./util");
program
    .version('0.0.1')
    .option('-p, --project <file>', 'path to tsconfig.json');
program.on('--help', () => {
    console.log(`
  $ tscpath -p tsconfig.json
`);
});
program.parse(process.argv);
const { project } = program;
if (!project) {
    throw new Error('--project must be specified');
}
console.log("***tsc-alias starting***");
const configFile = path_1.resolve(process.cwd(), project);
const { baseUrl, outDir, paths } = util_1.loadConfig(configFile);
if (!baseUrl) {
    throw new Error('compilerOptions.baseUrl is not set');
}
if (!paths) {
    throw new Error('compilerOptions.paths is not set');
}
if (!outDir) {
    throw new Error('compilerOptions.outDir is not set');
}
const configDir = normalizePath(path_1.dirname(configFile));
const confDirParentFolderName = path_1.basename(configDir);
const basePath = normalizePath(util_1.getPathThatEndsUp(util_1.walk(normalizePath(path_1.normalize(`${configDir}/${outDir}`)), confDirParentFolderName), confDirParentFolderName));
const outPath = normalizePath(configDir + '/' + outDir);
const aliases = Object.keys(paths)
    .map((alias) => ({
    prefix: alias.replace(/\*$/, ''),
    paths: paths[alias].map((p) => p.replace(/\*$/, '')
        .replace('ts', 'js')),
}))
    .filter(({ prefix }) => prefix);
const requireRegex = /(?:import|require)\(['"]([^'"]*)['"]\)/g;
const importRegex = /(?:import|from) ['"]([^'"]*)['"]/g;
const replaceImportStatement = (orig, file, aliasPrefix, aliasPath) => {
    const requiredModule = orig.split(/"|'/)[1];
    const index = orig.indexOf(aliasPrefix);
    const isAlias = requiredModule === aliasPrefix;
    if (index > -1 && isAlias) {
        const absoluteAliasPath = normalizePath(`${basePath}/${aliasPath}`);
        const relativeAliasPath = normalizePath(path_1.relative(path_1.dirname(file), absoluteAliasPath));
        const modulePath = orig.substring(0, index) +
            relativeAliasPath +
            orig.substring(index + aliasPrefix.length);
        return modulePath;
    }
    return orig;
};
const replaceAlias = (text, file) => {
    for (const alias of aliases) {
        text = text
            .replace(requireRegex, (orig) => replaceImportStatement(orig, file, alias.prefix, alias.paths[0]))
            .replace(importRegex, (orig) => replaceImportStatement(orig, file, alias.prefix, alias.paths[0]));
    }
    return text;
};
const files = globby_1.sync(`${outPath}/**/*.{js,jsx,ts,tsx}`, {
    dot: true,
    noDir: true,
}).map((x) => path_1.resolve(x));
const flen = files.length;
for (let i = 0; i < flen; i += 1) {
    const file = files[i];
    const text = fs_1.readFileSync(file, 'utf8');
    const newText = replaceAlias(text, file);
    if (text !== newText) {
        fs_1.writeFileSync(file, newText, 'utf8');
    }
}
//# sourceMappingURL=index.js.map